//Copyright (c) 2014, Jesús Martín Berlanga. All rights reserved. 
//Distributed under the BSD licence. Read "com/jme3/ai/license.txt".
package com.jme3.ai.agents;

import com.jme3.ai.agents.util.GameEntityExceptions;

/**
 *
 * Class container for exceptions related to agent.
 *
 * @author Tihomir Radosavljević
 * @author Jesús Martín Berlanga
 * @version 1.0.1
 */
public class AgentExceptions {

    /**
     * This exception is thrown If it has been tried to instantiate an agent
     * with illegal arguments.
     */
    public static class IllegalAgentException extends RuntimeException {

        public IllegalAgentException(String message) {
            super(message);
        }
    }

    /**
     * Agent does not have initiliazed attribute.
     */
    public static class AgentAttributeNotFound extends GameEntityExceptions.GameEntityAttributeNotFound {

        public AgentAttributeNotFound(Agent agent, String message) {
            super(agent, message);
        }
    }

    public static class TeamNotFoundException extends AgentAttributeNotFound {

        public TeamNotFoundException(Agent agent) {
            super(agent, "a designated team");
        }
    }

    public static class InventoryNotFoundException extends AgentAttributeNotFound {

        public InventoryNotFoundException(Agent agent) {
            super(agent, "an inventory");
        }
    }

    public static class WeaponNotFoundException extends AgentAttributeNotFound {

        public WeaponNotFoundException(Agent agent) {
            super(agent, "a weapon");
        }
    }

    public static class InvalidNeighborhoodDistanceException extends IllegalAgentException{

        public InvalidNeighborhoodDistanceException(String message) {
            super(message);
        }
    }
    
    public static class MaxAgentsException extends RuntimeException {

        public MaxAgentsException() {
            super("There are more agents than the framework supports.");
        } 
    }
}
