//Copyright (c) 2014, Jesús Martín Berlanga. All rights reserved. 
//Distributed under the BSD licence. Read "com/jme3/ai/license.txt".
package com.jme3.ai.agents.behaviors;

import com.jme3.ai.agents.Agent;
import com.jme3.ai.agents.behaviors.BehaviorExceptions.AgentNotIncludedException;
import com.jme3.renderer.RenderManager;
import com.jme3.renderer.ViewPort;
import com.jme3.scene.Spatial;
import com.jme3.scene.control.AbstractControl;

/**
 * Base class for agent behaviors.
 *
 * @author Tihomir Radosavljević
 * @author Jesús Martín Berlanga
 * @version 1.2.1
 */
public abstract class Behavior extends AbstractControl {

    /**
     * Agent to whom behavior belongs.
     */
    protected Agent agent;

    /**
     * Constructor for behavior that doesn't have any special spatial during
     * execution.
     *
     * @param agent to whom behavior belongs
     * @throws AgentNotIncludedException if agent is null
     */
    public Behavior(Agent agent) {
        if (agent == null) {
            throw new BehaviorExceptions.AgentNotIncludedException();
        }
        this.agent = agent;
        this.spatial = agent.getSpatial();
    }

    /**
     * Constructor for behavior that has spatial during execution.
     *
     * @param agent to whom behavior belongs
     * @param spatial which is active during execution
     * @see Behavior#Behavior(com.jme3.ai.agents.Agent)
     * @throws AgentNotIncludedException if agent is null
     */
    public Behavior(Agent agent, Spatial spatial) {
        if (agent == null) {
            throw new BehaviorExceptions.AgentNotIncludedException();
        }
        this.agent = agent;
        this.spatial = spatial;
    }

    @Override
    protected void controlRender(RenderManager rm, ViewPort vp) {
    }
}
