//Copyright (c) 2014, Jesús Martín Berlanga. All rights reserved. 
//Distributed under the BSD licence. Read "com/jme3/ai/license.txt".
package com.jme3.ai.agents.behaviors;

import com.jme3.ai.agents.Agent;

/**
 * Class container for exceptions related to behaviors.
 *
 * @author Tihomir Radosavljević
 * @author Jesús Martín Berlanga
 * @version 1.0.0
 */
public class BehaviorExceptions {

    /**
     * This exception is thrown if it has been tried to instantiate a behavior
     * with illegal arguments.
     */
    public static class BehaviorException extends RuntimeException {

        public BehaviorException(String message) {
            super(message);
        }

        public BehaviorException(Agent agent, String message, float value) {
            super(agent + message + value + '.');
        }
    }

    public static class AgentNotIncludedException extends BehaviorException {

        public AgentNotIncludedException() {
            super("You can not instantiate a behaviour without an agent.");
        }
    }

    public static class NullBehaviorException extends BehaviorException {

        public NullBehaviorException() {
            super("Behaviour has not been initialized");
        }

        public NullBehaviorException(String message) {
            super(message);
        }
    }

    /**
     * This exception is thrown if the behavior tries to call null target.
     */
    public static class TargetNotFoundException extends BehaviorException {

        public TargetNotFoundException() {
            super("Target can not be null");
        }
    }
}
