//Copyright (c) 2014, Jesús Martín Berlanga. All rights reserved. 
//Distributed under the BSD licence. Read "com/jme3/ai/license.txt".
package com.jme3.ai.agents.behaviors.npc.steering;

import com.jme3.ai.agents.Agent;
import com.jme3.ai.agents.behaviors.BehaviorExceptions;
import com.jme3.math.Vector3f;
import com.jme3.scene.Spatial;

/**
 * Pursuit is similar to seek except that the quarry (target) is another moving
 * character. Effective pursuit requires a prediction of the target’s future
 * position.
 *
 * @author Jesús Martín Berlanga
 * @version 1.2
 */
public class PursuitBehavior extends SeekBehavior {

    /**
     * @see SeekBehavior#SeekBehavior(com.jme3.ai.agents.Agent,
     * com.jme3.ai.agents.Agent)
     * @throws BehaviorExceptions.TargetNotFoundException If target is null
     */
    public PursuitBehavior(Agent agent, Agent target) {
        super(agent, target);
        this.validateTarget(target);
    }

    /**
     * @see SeekBehavior#SeekBehavior(com.jme3.ai.agents.Agent,
     * com.jme3.ai.agents.Agent, com.jme3.scene.Spatial)
     * @throws BehaviorExceptions.TargetNotFoundException If target is null
     */
    public PursuitBehavior(Agent agent, Agent target, Spatial spatial) {
        super(agent, target, spatial);
        this.validateTarget(target);
    }

    private void validateTarget(Agent target) {
        if (target == null) {
            throw new BehaviorExceptions.TargetNotFoundException();
        }
    }

    /**
     * @see AbstractStrengthSteeringBehavior#calculateRawSteering()
     */
    @Override
    protected Vector3f calculateRawSteering() {
        //See how far ahead we need to leed
        Vector3f projectedLocation = this.getTarget().getPredictedPosition();

        //Seek behaviour
        Vector3f desierdVel = projectedLocation.subtract(this.agent.getLocalTranslation());

        Vector3f aVelocity = this.agent.getVelocity();

        if (aVelocity == null) {
            aVelocity = new Vector3f();
        }

        return desierdVel.subtract(aVelocity);
    }
}
