//Copyright (c) 2014, Jesús Martín Berlanga. All rights reserved. 
//Distributed under the BSD licence. Read "com/jme3/ai/license.txt".
package com.jme3.ai.agents.behaviors.npc.steering;

import com.jme3.ai.agents.Agent;
import com.jme3.ai.agents.behaviors.BehaviorExceptions.BehaviorException;

/**
 * Class container for exceptions related to steering behavior.
 *
 * @author Tihomir Radosavljević
 * @author Jesús Martín Berlanga
 * @version 1.0.0
 */
public class SteeringExceptions {

    /**
     * Main generic exception related for all illegal arguments for steering
     * behaviors.
     */
    public static class SteeringBehaviorException extends BehaviorException {

        public SteeringBehaviorException(Agent agent, String message, float value) {
            super(agent, message, value);
        }

        public SteeringBehaviorException(String message) {
            super(message);
        }
    }

    /**
     * This exception is thrown when value is not set in the [0, 1] interval.
     */
    public static class IllegalIntervalException extends SteeringBehaviorException {

        public IllegalIntervalException(String factorName, float value) {
            super("The " + factorName + " factor value must be contained in the [0,1] interval. The current value is " + value + '.');
        }
    }

    /**
     * This exception is thrown when negative value is given to steering
     * behaviour.
     */
    public static class NegativeValueException extends SteeringBehaviorException {

        public NegativeValueException(String message) {
            super(message);
        }

        public NegativeValueException(String message, float value) {
            super(message + " You inputed " + value + '.');
        }
    }

    public static class InvalidAreaException extends SteeringBehaviorException {

        public InvalidAreaException(String message) {
            super(message);
        }
    }

    public static class PathFollowInsufficientPointsException extends BehaviorException {

        public PathFollowInsufficientPointsException(String message, float value) {
            super(message + " You inputed " + value + '.');
        }
    }

    public static class WallApproachWithoutWallException extends BehaviorException {

        public WallApproachWithoutWallException(String message) {
            super(message);
        }
    }
}
