//Copyright (c) 2014, Jesús Martín Berlanga. All rights reserved. 
//Distributed under the BSD licence. Read "com/jme3/ai/license.txt".
package com.jme3.ai.agents.util;

/**
 * Class container for exceptions related to game entities.
 *
 * @author Tihomir Radosavljević
 * @author Jesús Martín Berlanga
 * @version 1.1.0
 */
public class GameEntityExceptions {

    /**
     * Main exception for all Game entity related exceptions.
     */
    public static class GameEntityException extends RuntimeException {

        public GameEntityException(String message) {
            super(message);
        }
    }

    /**
     * Attribute is not initialized in game entity.
     */
    public static class GameEntityAttributeNotFound extends GameEntityException {

        public GameEntityAttributeNotFound(GameEntity gameEntity, String message) {
            super(gameEntity + " doesn't have " + message + '.');
        }
    }

    /**
     * Game entity has negative radius.
     */
    public static class NegativeRadiusException extends GameEntityException {

        public NegativeRadiusException(String message) {
            super(message);
        }
    }
    
    public static class MaxGameEntitiesException extends RuntimeException {

        public MaxGameEntitiesException() {
            super("There are more game entities than the framework supports.");
        } 
    }
}
